@@ -121,7 +121,7 @@ class EInfinitySurface(Surface):
                 field=lambda_squared.parent()
             else:
                 self._l=field(lambda_squared)
-        Surface.__init__(self,field, ZZ.zero(), finite=False)
+        super().__init__(field, ZZ.zero(), finite=False, mutable=False)
 
     def _repr_(self):
         r"""
@@ -267,7 +267,7 @@ class TFractalSurface(Surface):
 
         base_label=self.polygon_labels()._cartesian_product_of_elements((self._words(''), 0))
 
-        Surface.__init__(self, field, base_label, finite=False)
+        super().__init__(field, base_label, finite=False, mutable=False)
 
     def _repr_(self):
         return "The T-fractal surface with parameters w=%s, r=%s, h1=%s, h2=%s"%(
@@ -496,7 +496,7 @@ class SimilaritySurfaceGenerators:
             sage: P = E(R.random_element()) # optional: exactreal
             sage: S = similarity_surfaces.billiard(P); S # optional: exactreal
             ConeSurface built from 2 polygons
-            sage: TestSuite(S).run() # optional: exactreal
+            sage: TestSuite(S).run() # long time (6s), optional: exactreal
 
         """
         if not isinstance(P, Polygon):
@@ -1163,7 +1163,7 @@ class TranslationSurfaceGenerators:
             sage: C = translation_surfaces.cathedral(K.gen(), R.random_element([0.1, 0.2])) # optional: exactreal
             sage: C.stratum() # optional: exactreal
             H_4(2^3)
-            sage: TestSuite(C).run() # optional: exactreal
+            sage: TestSuite(C).run() # long time (6s), optional: exactreal
         """
         ring = Sequence([a,b]).universe()
         if isinstance(ring, type):
